
//shading:         gouraud
//lighting model:  blinn
//lighting type:   directional

//Original code from Jason Zink [Dual Paraboloid Mapping in the Vertex Shader]
//(http://www.gamedev.net/reference/articles/article2308.asp)

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tWV: WORLDVIEW;
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP : WORLDVIEWPROJECTION;

float viewParms <string uiname="View Direction";> = {0.5};	// ***.z = 1 for forward hemisphere, = -1 for backward hemisphere

//light properties
float3 lDir <string uiname="Light Direction";> = {0, -5, 2};       //light direction in world space
float4 lAmb  : COLOR <String uiname="Ambient Color";>  = {0.15, 0.15, 0.15, 1};
float4 lDiff : COLOR <String uiname="Diffuse Color";>  = {0.85, 0.85, 0.85, 1};
float4 lSpec : COLOR <String uiname="Specular Color";> = {0.35, 0.35, 0.35, 1};
float lPower <String uiname="Power"; float uimin=0.0;> = 25.0;     //shininess of specular highlight

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 tColor <string uiname="Color Transform";>;

struct vs2ps
{
    float4 Pos      : POSITION;
    float4 TexCd    : TEXCOORD0;
    float4 Diffuse  : COLOR0;
    float4 Specular : COLOR1;
    float  z_value  : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 PosO  : POSITION,
    float3 NormO : NORMAL,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //inverse light direction in view space
    float3 LightDirV = normalize(-mul(lDir, tV));

    //normal in view space
    float3 NormV = normalize(mul(NormO, tWV));

    //view direction = inverse vertexposition in viewspace
    float4 PosV = mul(PosO, tWV);
    float3 ViewDirV = normalize(-PosV);

    //halfvector
    float3 H = normalize(ViewDirV + LightDirV);

    //compute blinn lighting
    float3 shades = lit(dot(NormV, LightDirV), dot(NormV, H), lPower);

    float4 diff = lDiff * shades.y;
    diff.a = 1;
    float4 spec = lSpec * shades.z;
    spec.a = 1;

    //position (projected)
    Out.Pos  = mul(PosV, tP);
    Out.TexCd = mul(TexCd, tTex);
    Out.Diffuse = diff + lAmb;
    Out.Specular = spec;
    
     float direction = viewParms;		// this determines the front or back hemisphere

   	// divide by w to normalize
        Out.Pos = mul(PosO, tWVP);
        Out.Pos.z = Out.Pos.z * direction;	// set z-values to forward or backward
        float L = length( Out.Pos.xyz );	// determine the distance between (0,0,0) and the vertex
        Out.Pos = Out.Pos  / L;		        // divide the vertex position by the distance
        Out.z_value = Out.Pos.z;		// remember which hemisphere the vertex is in
        Out.Pos .z = Out.Pos.z + 1;		// add the reflected vector to find the normal vector
        Out.Pos.x = Out.Pos.x / Out.Pos.z;	// divide x coord by the new z-value
        Out.Pos.y = Out.Pos.y / Out.Pos.z;	// divide y coord by the new z-value
        Out.Pos.z = L / 500;		        // set a depth value for correct z-buffering
	Out.Pos.w = 1;

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0
    clip(In.z_value + 0.5) ;
    
    float4 col = tex2D(Samp, In.TexCd);
    col.rgb *= In.Diffuse + In.Specular;
    return mul(col, tColor);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique FishEye
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader = compile ps_1_4 PS();
    }
}

technique TGouraudDirectionalFF
{
    pass P0
    {
        //transformations
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //material
        MaterialAmbient  = {1, 1, 1, 1};
        MaterialDiffuse  = {1, 1, 1, 1};
        MaterialSpecular = {1, 1, 1, 1};
        MaterialPower    = (lPower);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere

        //lighting
        LightEnable[0] = TRUE;
        Lighting       = TRUE;
        SpecularEnable = TRUE;

        LightType[0]     = DIRECTIONAL;
        LightAmbient[0]  = (lAmb);
        LightDiffuse[0]  = (lDiff);
        LightSpecular[0] = (lSpec);
        LightDirection[0] = (lDir);

        //shading
        ShadeMode = GOURAUD;
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
